import java.io.*;
import java.util.*;
import java.util.zip.*;

public class NGramDBFromText {
    public NGramDBFromText() {
    }

    private static void addGram(TreeMap<String, Integer> hm, String gram) {
        if(!hm.containsKey(gram))
            hm.put(gram, 1);
        else
            hm.put(gram, hm.get(gram)+1);
    }

    private static long extractGramsFromFile(File inFile, TreeMap<String,
                                             Integer> hm, int gramSize) throws
            Exception {
        BufferedReader brIn = new BufferedReader(new FileReader(inFile));
        String sLine;
        int total = 0;
        while ((sLine = brIn.readLine()) != null) {
            //TODO: Replace this line with real tokenization code
            String[] words = ("<S> " + sLine + " </S>").split(" +");
            total += words.length;
            for (int i = 0; i < words.length; i++) {
                String gram = words[i];
                addGram(hm, gram);
                for (int j = 1; (j < gramSize && i + j < words.length); j++) {
                    gram += " " + words[i + j];
                    addGram(hm, gram);
                }
            }
        }
        brIn.close();
        return total;
    }

    private static int getGramCount(String s) {
        int c = 1;
        int last = -1;
        while((last = s.indexOf(" ", last+1))>0) c++;
        return c;
    }

    private static void createNGramDB(String inDirectory, String outDirectory, int maxGramSize)
    throws Exception {
        File fIn = new File(inDirectory);
        File [] fInFiles = fIn.listFiles();
        TreeMap<String, Integer> grams = new TreeMap<String, Integer>();
        String [] firstGrams = new String[maxGramSize];
        long total = 0;
        for(int i=0; i<fInFiles.length; i++) {
            total += extractGramsFromFile(fInFiles[i], grams, maxGramSize);
        }
        Iterator<String> it = grams.keySet().iterator();
        File outDir = new File(outDirectory);
        GZIPOutputStream [] outStreams = new GZIPOutputStream[maxGramSize]; //1-grams go in outFiles[0]

        for (int i = 0; i < maxGramSize; i++) {
            String dirName = outDirectory + "\\" + (i + 1) + "gms";
            (new File(dirName)).mkdir();
            if (i == 0)
                outStreams[i] = new GZIPOutputStream(new FileOutputStream(dirName + "\\vocab.gz"));
            else
                outStreams[i] = new GZIPOutputStream(new FileOutputStream(dirName + "\\" + (i+1) + "gm-0000.gz"));
        }
        outDir.mkdir();
        while(it.hasNext()) {
            String gram = it.next();
            int count = grams.get(gram);
            int numGrams = getGramCount(gram);
            if(firstGrams[numGrams-1] == null)
                firstGrams[numGrams-1] = gram;
            outStreams[numGrams - 1].write((gram + "\t" + count + "\n").getBytes());
        }

        for(int i=0; i< maxGramSize; i++) {
            outStreams[i].close();
            if(i==0) continue;
            BufferedWriter bwOut = new BufferedWriter(new FileWriter(outDirectory +
                    "\\" + (i + 1) + "gms\\" + (i + 1) + "gm.idx"));
            bwOut.write((i+1) + "gm-0000.gz" + firstGrams[i]);
            bwOut.close();
        }
        BufferedWriter bwOut = new BufferedWriter(new FileWriter(outDirectory +
                "\\1gms\\total"));
        bwOut.write(Long.toString(total));
        bwOut.close();
    }

    public static void main(String[] args) throws Exception {
        String inDir = null;
        String outDir = null;
        int gramSize = 0;
        try {
            inDir = args[0];
            outDir = args[1];
            gramSize = Integer.parseInt(args[2]);
        }
        catch(Exception e) {
            System.out.println("Usage: NGramDBFromText <inDirectory> <outDirectory> <maxGramSize>");
            return;
        }
        if(gramSize == 0)
            return;
        createNGramDB(inDir, outDir, gramSize);
    }
}
