/*
 * OpenNLPProvider.java
 *
 * Copyright (c) 2007-2008 Colin Bayer, Douglas Downey, Oren Etzioni,
 *   University of Washington Computer Science and Engineering
 * 
 * See the file "COPYING" in the root directory of this distribution for
 * full licensing information.
 */

import opennlp.tools.tokenize.*;
import opennlp.tools.sentdetect.*;

import java.lang.reflect.*;

public class OpenNLPProvider {
	public static SentenceDetector createSentenceDetector() {
		try {
			if (mSentenceDetectorModelPath == null)
				return mSentenceDetectorClass.getConstructor().newInstance();
			else
				return mSentenceDetectorClass.getConstructor(String.class).newInstance(mSentenceDetectorModelPath);
		} catch (Exception e) {
			System.err.println("Error instantiating sentence detector: " + e);
			return null;
		}
	}

	public static Tokenizer createTokenizer() {
		try {
			if (mTokenizerModelPath == null)
				return mTokenizerClass.getConstructor().newInstance();
			else
				return mTokenizerClass.getConstructor(String.class).newInstance(mTokenizerModelPath);
		} catch (Exception e) {
			System.err.println("Error instantiating sentence detector: " + e);
			return null;
		}
	}

	public static void setSentenceDetectorClass(Class<? extends SentenceDetector> aClass) {
		mSentenceDetectorClass = aClass;
	}

	public static void setTokenizerClass(Class<? extends Tokenizer> aClass) {
		mTokenizerClass = aClass;
	}

	public static void setSentenceDetectorModelPath(String aPath) {
		mSentenceDetectorModelPath = aPath;
	}

	public static void setTokenizerModelPath(String aPath) {
		mTokenizerModelPath = aPath;
	}

	private static String mSentenceDetectorModelPath;
	private static String mTokenizerModelPath;

	private static Class<? extends SentenceDetector> mSentenceDetectorClass;
	private static Class<? extends Tokenizer> mTokenizerClass;
}
